#include "common.h"

/* ------------------------------------------------------------------------ */

BOOL DeleteProfile(const char *profile)
{
	HRESULT hResult;
	char keyname[256 + PROFNAME_MAX];

	if(strlen(profile) >= PROFNAME_MAX)
		return FALSE;
	strcpy(keyname, REGNAME_ROOT);
	strcat(keyname, "\\");
	strcat(keyname, profile);

	hResult = RegDeleteKey(HKEY_LOCAL_MACHINE, keyname);
	return (hResult == ERROR_SUCCESS);
}

/* ------------------------------------------------------------------------ */

BOOL SaveProfile(const char *profile, const profiledata *profdata)
{
	HKEY hProfile;
	HRESULT hResult;
	DWORD dwProto, dwTmp;
	char keyname[256 + PROFNAME_MAX];

	if(strlen(profile) >= PROFNAME_MAX)
		return FALSE;
	strcpy(keyname, REGNAME_ROOT);
	strcat(keyname, "\\");
	strcat(keyname, profile);

	hResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyname, 0, NULL, 
							 REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
							 &hProfile, &dwTmp);
	if(hResult != ERROR_SUCCESS)
		return FALSE;

	if(profdata->use_cmdline) {
		dwTmp = 1;
		/* same command line only */
		if(RegSetValueEx(hProfile, REGNAME_CMDLINE, 0, REG_SZ, 
						 profdata->cmdline, 
						 strlen(profdata->cmdline) + 1) != ERROR_SUCCESS
		|| RegSetValueEx(hProfile, REGNAME_USECMDLINE, 0, REG_DWORD, 
						  (LPSTR)&dwTmp, sizeof(DWORD)) != ERROR_SUCCESS) {
			RegCloseKey(hProfile);
			return FALSE;
		}
	} else {
		dwProto = (profdata->ssh_proto == 2 ? 2 : 1);
		dwTmp = 0;
		if(RegSetValueEx(hProfile, REGNAME_USECMDLINE, 0, REG_DWORD, 
						  (LPSTR)&dwTmp, sizeof(DWORD)) != ERROR_SUCCESS
		|| RegSetValueEx(hProfile, REGNAME_USERNAME, 0, REG_SZ, 
						 profdata->username, 
						 strlen(profdata->username) + 1) != ERROR_SUCCESS
		|| RegSetValueEx(hProfile, REGNAME_HOSTNAME, 0, REG_SZ, 
						  profdata->hostname, 
						  strlen(profdata->hostname) + 1) != ERROR_SUCCESS
		|| RegSetValueEx(hProfile, REGNAME_KEYPATH, 0, REG_SZ, 
						  profdata->keypath, 
						  strlen(profdata->keypath) + 1) != ERROR_SUCCESS
		|| RegSetValueEx(hProfile, REGNAME_SSHPROTOCOL, 0, REG_DWORD, 
						  (LPSTR)&dwProto, sizeof(DWORD)) != ERROR_SUCCESS) {
			RegCloseKey(hProfile);
			return FALSE;
		}
		if(profdata->use_session) {
			dwTmp = 1;
			if(RegSetValueEx(hProfile, REGNAME_SESSIONNAME, 0, REG_SZ, 
							 profdata->session, 
							 strlen(profdata->session) + 1) != ERROR_SUCCESS
			|| RegSetValueEx(hProfile, REGNAME_USESESSION, 0, REG_DWORD, 
							  (LPSTR)&dwTmp, sizeof(DWORD)) != ERROR_SUCCESS) {
				RegCloseKey(hProfile);
				return FALSE;
			}
		} else {
			dwTmp = 0;
			if(RegSetValueEx(hProfile, REGNAME_USESESSION, 0, REG_DWORD, 
							  (LPSTR)&dwTmp, sizeof(DWORD)) != ERROR_SUCCESS) {
				RegCloseKey(hProfile);
				return FALSE;
			}
		}
		dwTmp = profdata->port;
		if(RegSetValueEx(hProfile, REGNAME_PORTNUMBER, 0, REG_DWORD, 
						  (LPSTR)&dwTmp, sizeof(DWORD)) != ERROR_SUCCESS) {
			RegCloseKey(hProfile);
			return FALSE;
		}

	}
	RegCloseKey(hProfile);
	return TRUE;
}

/* ------------------------------------------------------------------------ */

BOOL IsProfileExists(const char *profile)
{
	HRESULT hResult;
	HKEY hProfile;
	char keyname[256 + PROFNAME_MAX];

	if(strlen(profile) >= PROFNAME_MAX)
		return FALSE;
	strcpy(keyname, REGNAME_ROOT);
	strcat(keyname, "\\");
	strcat(keyname, profile);

	hResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname, 0, KEY_READ, 
							 &hProfile);
	if(hResult != ERROR_SUCCESS)
		return FALSE;
	RegCloseKey(hProfile);
	return TRUE;
}

/* ------------------------------------------------------------------------ */

BOOL LoadProfile(const char *profile, profiledata *profdata)
{
	HKEY hProfile;
	HRESULT hResult;
	DWORD dwProto, dwTmp, dwType, dwSize;
	char keyname[256 + PROFNAME_MAX];

	if(strlen(profile) >= PROFNAME_MAX)
		return FALSE;
	strcpy(keyname, REGNAME_ROOT);
	strcat(keyname, "\\");
	strcat(keyname, profile);

	hResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname, 0, KEY_READ, 
						   &hProfile);
	if(hResult != ERROR_SUCCESS)
		return FALSE;

	/* version 2, optional */
	dwSize = sizeof(dwTmp);
	/* note: no error check */
	if(RegQueryValueEx(hProfile, REGNAME_USECMDLINE, NULL, &dwType, 
					(PBYTE)&dwTmp, &dwSize) != ERROR_SUCCESS)
		profdata->use_cmdline = FALSE;
	else if(dwType != REG_DWORD) {
		RegCloseKey(hProfile);
		return FALSE;
	} else if(dwTmp != 0) {
		/* only command line */
		profdata->use_cmdline = TRUE;
		if(dwSize = sizeof(profdata->cmdline),
			(RegQueryValueEx(hProfile, REGNAME_CMDLINE, NULL, &dwType, 
						    (PBYTE)&(profdata->cmdline), &dwSize) != ERROR_SUCCESS
							|| dwType != REG_SZ)) {
			RegCloseKey(hProfile);
			return FALSE;
		}
	} else {
		profdata->use_cmdline = FALSE;
	}
	if(!profdata->use_cmdline) {
		/* version 1 */
		if((dwSize = sizeof(profdata->username),
			(RegQueryValueEx(hProfile, REGNAME_USERNAME, NULL, &dwType, 
						    (PBYTE)&(profdata->username), &dwSize) != ERROR_SUCCESS
			|| dwType != REG_SZ))
		|| (dwSize = sizeof(profdata->hostname),
			(RegQueryValueEx(hProfile, REGNAME_HOSTNAME, NULL, &dwType, 
							(PBYTE)&(profdata->hostname), &dwSize) != ERROR_SUCCESS
			|| dwType != REG_SZ))
		|| (dwSize = sizeof(profdata->keypath),
			(RegQueryValueEx(hProfile, REGNAME_KEYPATH, NULL, &dwType, 
							(PBYTE)&(profdata->keypath), &dwSize) != ERROR_SUCCESS
			|| dwType != REG_SZ))
		|| (dwSize = sizeof(dwProto),
			(RegQueryValueEx(hProfile, REGNAME_SSHPROTOCOL, NULL, &dwType, 
							(PBYTE)&dwProto, &dwSize) != ERROR_SUCCESS
			|| dwType != REG_DWORD))) {
			RegCloseKey(hProfile);
			return FALSE;
		}
		profdata->ssh_proto = (dwProto == 2 ? 2 : 1);

		/* version 2, optional field */
		dwSize = sizeof(dwTmp);
		/* note: no error check */
		if(RegQueryValueEx(hProfile, REGNAME_USESESSION, NULL, &dwType, 
						(PBYTE)&dwTmp, &dwSize) != ERROR_SUCCESS)
			profdata->use_session = FALSE;
		else if(dwType != REG_DWORD) {
			/* invalid type, dropped */
			RegCloseKey(hProfile);
			return FALSE;
		} else {
			profdata->use_session = (dwTmp != 0);
		}
		if(profdata->use_session) {
			dwSize = sizeof(profdata->session);
			if(RegQueryValueEx(hProfile, REGNAME_SESSIONNAME, NULL, &dwType, 
							(PBYTE)&(profdata->session), &dwSize) != ERROR_SUCCESS
			|| dwType != REG_SZ) {
				RegCloseKey(hProfile);
				return FALSE;
			}
		}
		dwSize = sizeof(dwTmp);
		/* note: no error check */
		if(RegQueryValueEx(hProfile, REGNAME_PORTNUMBER, NULL, &dwType, 
						(PBYTE)&dwTmp, &dwSize) != ERROR_SUCCESS)
			profdata->port = 0;
		else if(dwType != REG_DWORD) {
			/* invalid type, dropped */
			RegCloseKey(hProfile);
			return FALSE;
		} else {
			profdata->port = (USHORT)(dwTmp & 0xFFFF);
		}
	}
	RegCloseKey(hProfile);
	
	return TRUE;
}

/* ------------------------------------------------------------------------ */

int BuildOptions(char *optbuf, int maxlen, const profiledata *profdata)
{
	char *p = optbuf;
	int issp;
	if(profdata->use_cmdline) {
		if(maxlen == 0)
			return strlen(profdata->cmdline) + 1;
		strncpy(optbuf, profdata->cmdline, maxlen);
		optbuf[maxlen - 1] = '\0';
		return 0;
	}
#define APPENDOPT(arg, optname)										\
	do {															\
		const char *opt = arg;										\
		issp = 0;													\
		if(p > optbuf) {											\
			if(p - optbuf < maxlen)									\
				*p = ' ';											\
			p++;													\
		}															\
		if(p - optbuf + (int)strlen(opt) < maxlen)					\
			strcpy(p, opt);											\
		p += strlen(opt);											\
		issp = ((strchr(optname, ' ')) != NULL || *(optname) == '\0');	\
		if(issp) {													\
			if(p - optbuf < maxlen)									\
				*p = '\"';											\
			p++;													\
		}															\
		if(p - optbuf + (int)strlen(optname) < maxlen)				\
			strcpy(p, optname);										\
		p += strlen(optname);										\
		if(issp) {													\
			if(p - optbuf < maxlen)									\
				*p = '\"';											\
			p++;													\
		}															\
	} while(0)
	/* -load session */
	if(profdata->use_session)
		APPENDOPT("-load ", profdata->session);
	/* -P port */
	if(profdata->port) {
		char tmpbuf[16];
		sprintf(tmpbuf, "%u", profdata->port);
		APPENDOPT("-P ", tmpbuf);
	}
	/* -l username */
	APPENDOPT("-l ", profdata->username);
	/* -1 -2 */
	if(profdata->ssh_proto == 2) {
		const char *opt = " -2";
		if(p - optbuf + (int)strlen(opt) < maxlen)
			strcpy(p, opt);
		p += strlen(opt);
	} else {
		const char *opt = " -1";
		if(p - optbuf + (int)strlen(opt) < maxlen)
			strcpy(p, opt);
		p += strlen(opt);
	}
	/* -i keyfile */
	APPENDOPT("-i ", profdata->keypath);
	/* hostname */
	if(p - optbuf < maxlen)
		*p = ' ';
	p++;
	issp = ((strchr(profdata->hostname, ' ')) != NULL
		  || profdata->hostname[0] == '\0');
	if(issp) {
		if(p - optbuf < maxlen)
			*p = '\"';
		p++;
	}
	if(p - optbuf + (int)strlen(profdata->hostname) < maxlen)
		strcpy(p, profdata->hostname);
	p += strlen(profdata->hostname);
	if(issp) {
		if(p - optbuf < maxlen)
			*p = '\"';
		p++;
	}
	if(p - optbuf < maxlen)
		*p = '\0';
	p++;
	return p - optbuf;
}

/* ------------------------------------------------------------------------ */
