#include "common.h"

/* ------------------------------------------------------------------------ */

static BOOL GetProfileName(HWND hWnd, char *profile)
{
	DWORD index;
	index = SendDlgItemMessage(hWnd, IDC_PROFLIST, LB_GETCURSEL, 0, 0);
	if(index == LB_ERR)
		return FALSE;
	if(SendDlgItemMessage(hWnd, IDC_PROFLIST, LB_GETTEXTLEN, index, 0)
		> PROFNAME_MAX - 1) {
		MessageBox(hWnd, "The profile name is too long.", "Error",
				   MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	SendDlgItemMessage(hWnd, IDC_PROFLIST, LB_GETTEXT, 
					   index, (LPARAM)profile);
	return TRUE;
}

/* ------------------------------------------------------------------------ */

static void OnProfileSelectionChanged(HWND hWnd)
{
	DWORD index;
	char profile[PROFNAME_MAX];
	profiledata profdata;

	index = SendDlgItemMessage(hWnd, IDC_PROFLIST, LB_GETCURSEL, 0, 0);
	if(index == LB_ERR) {
		/* IĂȂ */
		EnableWindow(GetDlgItem(hWnd, IDC_DELPROF), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_CONFPROF), FALSE);
		SendDlgItemMessage(hWnd, IDC_CONFDISP, WM_SETTEXT, 0, (LPARAM)"");
		return;
	}
	EnableWindow(GetDlgItem(hWnd, IDC_DELPROF), TRUE);
	EnableWindow(GetDlgItem(hWnd, IDC_CONFPROF), TRUE);
	if(GetProfileName(hWnd, profile)
		&& LoadProfile(profile, &profdata)) {
		char msgbuf[512 + USERNAME_MAX + HOSTNAME_MAX + KEYPATH_MAX];
		if(profdata.use_cmdline) {
			sprintf(msgbuf, "Command line: %s",
					profdata.cmdline);
		} else {
			sprintf(msgbuf, "Connect to: %s@%s\nProtocol version: SSH%d\nIdentity file: %s",
					profdata.username, profdata.hostname, profdata.ssh_proto, profdata.keypath);
		}
		SendDlgItemMessage(hWnd, IDC_CONFDISP, WM_SETTEXT, 0, (LPARAM)msgbuf);
	}
}

/* ------------------------------------------------------------------------ */

static BOOL UpdateProfileList(HWND hWnd)
{
	HRESULT hResult;
	HKEY hRoot;
	FILETIME ftTmp;
	DWORD dwIndex;
	HWND hListBox;

	hListBox = GetDlgItem(hWnd, IDC_PROFLIST);
	if(hListBox == NULL)
		return FALSE;
	
	/* make sure to clear items in the list box */
	SendMessage(hListBox, LB_RESETCONTENT, 0, 0);

	hResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, REGNAME_ROOT, 0, KEY_READ, &hRoot);
	if(hResult != ERROR_SUCCESS)
		return FALSE;
	for(dwIndex = 0; ; dwIndex++) {
		char pBuffer[PROFNAME_MAX];
		DWORD dwBufferSize = sizeof(pBuffer);
		hResult = RegEnumKeyEx(hRoot, dwIndex, pBuffer, &dwBufferSize, NULL, NULL, NULL, &ftTmp);
		if(hResult != ERROR_SUCCESS)
			break;
		SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)pBuffer);
	}
	RegCloseKey(hRoot);
	OnProfileSelectionChanged(hWnd);
	return TRUE;
}

/* ------------------------------------------------------------------------ */

/* on coming WM_COMMAND on IDD_MAINDLG */
static BOOL OnMainDlgCommand(HWND hWnd, WORD wId, WORD wAction)
{
	switch(wId) {
	case IDC_PROFLIST:
		if(wAction == LBN_SELCHANGE || wAction == LBN_SELCANCEL) {
			OnProfileSelectionChanged(hWnd);
		}
		return TRUE;
	case IDC_ADDPROF:
		if(EditProfile(NULL))
			UpdateProfileList(hWnd);
		return TRUE;
	case IDC_DELPROF: {
		char profile[PROFNAME_MAX];
		char msgbuf[512];
		if(GetProfileName(hWnd, profile)) {
			sprintf(msgbuf, "Are you really deleting the profile %s?",
					profile);
			if(MessageBox(hWnd, msgbuf, "Confirm: delete profile",
				MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
				DeleteProfile(profile);
				UpdateProfileList(hWnd);
			}
		}
		return TRUE;
	}
	case IDC_CONFPROF: {
		char profile[PROFNAME_MAX];
		if(GetProfileName(hWnd, profile)) {
			if(EditProfile(profile))
				UpdateProfileList(hWnd);
		}
		return TRUE;
	}
	case IDOK:
		/* TODO: ̂Ƃ "OK"  "Cancel" ɈႢ͂ȂA
		         ̃_CAOɉ炩̐ݒ荀ڂoꍇɂ
				 ̐ݒۑ邩ǂ̈Ⴂ܂͂
		 */
	case IDCANCEL:
		EndDialog(hWnd, 0);
		return TRUE;
	}
	return FALSE;
}

/* ------------------------------------------------------------------------ */

static BOOL CALLBACK MainDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
	case WM_INITDIALOG:
		UpdateProfileList(hWnd);
		return TRUE;
	case WM_COMMAND:
		return OnMainDlgCommand(hWnd, LOWORD(wParam), HIWORD(wParam));
	}
	return FALSE;
}

/* ------------------------------------------------------------------------ */

BOOL Configure()
{
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAINDLG), NULL, (DLGPROC)MainDlgProc);
	return TRUE;
}

/* ------------------------------------------------------------------------ */
